﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Silverlight.Map;

namespace SilverlightApplication37
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();
        }

        private void statesLayer_Imported(object sender, Infragistics.Silverlight.Map.MapLayerImportEventArgs e)
        {
            foreach (MapElement ele in (sender as MapLayer).Elements)
            {
                if (ele.Caption == "AK")
                {
                    MapData.SetValueData(ele, (double)5.0);
                }
            }
        }

        private MapGridData GetSampleData()
        {
            MapGridData data = new MapGridData()
            {
                new MapGridDataItem() { Name = "Jeff", Age=23 },
                new MapGridDataItem() { Name = "Joe", Age=28 },
                new MapGridDataItem() { Name = "Jan", Age=42 },
                new MapGridDataItem() { Name = "Jim", Age=56 },
                new MapGridDataItem() { Name = "Jen", Age=19 },
                new MapGridDataItem() { Name = "Jack", Age=64 }
            };
            return data;
        }

        private void map1_ElementClick(object sender, MapElementClickEventArgs e)
        {
            foreach (MapElement ele in (sender as XamWebMap).Layers[0].Elements)
            {
                if (MapData.GetValueData(ele) != null &&
                    MapData.GetValueData(ele).GetType() == typeof(MapGridData))
                {
                    MapData.SetValueData(ele, null);
                }
            }
            MapGridData mgd = GetSampleData();
            mgd.Title = e.Element.Caption;
            MapData.SetValueData(e.Element, mgd);
        }
    }

    public class MapGridDataItem
    {
        public string Name { get; set; }
        public int Age { get; set; }
    }

    public class MapGridData
        : ObservableCollection<MapGridDataItem>
    {
        public string Title { get; set; }
    }

    public class MapData
        : DependencyObject
    {
        public static readonly DependencyProperty ValueDataProperty = DependencyProperty.RegisterAttached("ValueData",
            typeof(object), typeof(MapData), new PropertyMetadata(null));

        public static object GetValueData(DependencyObject target)
        {
            return target.GetValue(MapData.ValueDataProperty);
        }

        public static void SetValueData(DependencyObject target, object value)
        {
            target.SetValue(MapData.ValueDataProperty, value);
        }
    }

    public class SelectableTemplate
    {
        public string MatchType { get; set; }
        public DataTemplate Template { get; set; }
    }

    public class MapTemplateSelector
        : ContentControl
    {

        public MapTemplateSelector()
        {
            SelectableTemplates = new ObservableCollection<SelectableTemplate>();
        }

        public readonly DependencyProperty ValueProperty = DependencyProperty.Register("Value", typeof(object),
            typeof(MapTemplateSelector), new PropertyMetadata(null, OnValuePropertyChanged));

        public object Value
        {
            get
            {
                return GetValue(ValueProperty);
            }
            set
            {
                SetValue(ValueProperty, value);
            }
        }

        public readonly DependencyProperty SelectableTemplatesProperty = DependencyProperty.Register("SelectableTemplates", typeof(ObservableCollection<SelectableTemplate>),
            typeof(MapTemplateSelector), new PropertyMetadata(null, OnSelectableTemplatesPropertyChanged));

        public ObservableCollection<SelectableTemplate> SelectableTemplates
        {
            get
            {
                return (ObservableCollection<SelectableTemplate>)GetValue(SelectableTemplatesProperty);
            }
            set
            {
                SetValue(SelectableTemplatesProperty, value);
            }
        }

        protected static void OnSelectableTemplatesPropertyChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            (sender as MapTemplateSelector).OnSelectableTemplatesChanged(e);
        }

        protected void UpdateTemplate(Type valueType)
        {
            DataTemplate template = null;
            if (valueType != null)
            {
                template = GetDataTemplate(valueType);
            }
            if (template != null)
            {
                Content = template.LoadContent();
            }
            else
            {
                Content = null;
            }
        }

        protected void OnSelectableTemplatesChanged(DependencyPropertyChangedEventArgs e)
        {
            Type valueType = null;
            if (Value != null)
            {
                valueType = Value.GetType();
            }
            UpdateTemplate(valueType);
        }

        protected static void OnValuePropertyChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            (sender as MapTemplateSelector).OnValueChanged(e);
        }

        protected void OnValueChanged(DependencyPropertyChangedEventArgs e)
        {
            Type valueType = null;
            if (e.NewValue != null)
            {
                valueType = e.NewValue.GetType();
            }
            UpdateTemplate(valueType);
            if (Content is FrameworkElement)
            {
                (Content as FrameworkElement).DataContext = e.NewValue;
            }
        }

        protected DataTemplate GetDataTemplate(Type valueType)
        {
            foreach (SelectableTemplate selectable in SelectableTemplates)
            {
                if (selectable.MatchType == valueType.Name)
                {
                    return selectable.Template;
                }
            }

            return null;
        }
    }
}
